;;###########################################################################
;; guidemap.lsp
;; Contains code to implement window objects and methods for guidemaps
;; (a graphical hypertext/hypercode interface for guiding data analysts) 
;; and for context-sensitive help.
;; Copyright (c) 1992-97 by Forrest W. Young
;;###########################################################################

(require "vista")

(defproto guidemap-proto '(slot-info instant-return) () workmap-proto)

(defmeth guidemap-proto :isnew (&rest args)
  (apply #'call-next-method args)
  (defmeth self :close ()
#+macintosh    (send *workmap* :size 490 280)
#+msdos        (send *workmap* :size 500 280)
#+X11          (send *workmap* :size 500 280)
    (send *vista* :guidemap nil)
    (call-next-method)))

(defmeth iconmap-proto :slot-info (&optional (list nil set))
  (if set (setf (slot-value 'slot-info) list))
  (slot-value 'slot-info))

(defmeth iconmap-proto :instant-return (&optional (logical nil set))
  (if set (setf (slot-value 'instant-return) logical))
  (slot-value 'instant-return))

(defmeth guidemap-proto :redraw ()
  (when (send self :gui)
   (let* ((n    (send self :num-icons))
          (icon-list (send self :icon-list))
          (icon-type nil)
          (redraw-order (send self :redraw-order))
          (draw-x-offset nil)
          (draw-y-offset nil)
          (x    (send self :x))
          (y    (send self :y))
          ) 
     (send self :start-buffering)
     (send self :erase-window)
     
     (when (> n 0)
           (dotimes (i n)
              (when (select (select (send self :connection-list) 
                                    (select redraw-order i)) 0)
                    (dotimes (j (length (select (send self :connection-list)
                                 (select redraw-order i))))
                     (send self :connect-icons 
                          (select redraw-order i)
                          (select (select (send self :connection-list)
                                 (select redraw-order i)) j))))
                    
                    (when (equal (send (select icon-list 
                                 (select redraw-order i)) :icon-state) "grey")
                    (send self :draw-bitmap 
                          (send (select icon-list (select redraw-order i))
                                :grey-icon)
                          (select x (select redraw-order i)) 
                          (select y (select redraw-order i))))
              (when (equal (send (select icon-list 
                           (select redraw-order i)) :icon-state) "normal")
                    (send self :draw-bitmap 
                          (send (select icon-list (select redraw-order i))
                                :icon)
                          (select x (select redraw-order i)) 
                          (select y (select redraw-order i))))
              (when (equal (send (select icon-list 
                           (select redraw-order i)) :icon-state) "selected")
                    (send self :draw-bitmap 
                          (send (select icon-list (select redraw-order i))
                                :hi-icon)
                          (select x (select redraw-order i)) 
                          (select y (select redraw-order i))))                    
              (setf draw-x-offset 13)
              (setf draw-y-offset 31)
              (setf icon-type 
                   (send (select icon-list (select redraw-order i)) 
                         :icon-type))
              (when (or (= 6 icon-type) (= 7 icon-type) (= 2 icon-type))
                    (setf draw-x-offset 22)
                    (setf draw-y-offset 12))
              (send self :draw-text (send 
                        (select icon-list (select redraw-order i)) :title)
                        (+ (select x (select redraw-order i)) draw-x-offset) 
                        (+ (select y (select redraw-order i)) draw-y-offset) 
                    1 1)
                    ))
     (send self :redraw-overlays)
     (send self :buffer-to-screen)
     )))

(defmeth guidemap-proto :connect-icons (icon-number-out icon-number-in 
                                                       &key new)
  (let* ((icon-out (select (send self :icon-list) icon-number-out))
         (x-out (send icon-out :x))
         (y-out (send icon-out :y))
         (icon-in  (select (send self :icon-list) icon-number-in))
         (x-in  (send icon-in  :x))
         (y-in  (send icon-in  :y))
         (center-out (ceiling (/ (send icon-out :width) 2)))
         (center-in  (ceiling (/ (send icon-in  :width) 2)))
         (below-out  (send icon-out :height))
         (constant   10)
         (right-move (- (+ x-in (* 2 center-in) constant) 
                        (+ x-out center-out)))
         (top-move   (- y-in y-out below-out 16))
         (left-move (+ center-in constant))
         )
    (when (> right-move 0) 
          (setf right-move (- x-in constant (+ x-out center-out)))
          (setf left-move (- left-move)))
    (when (<  (+ y-out below-out 15) y-in)
          (setf right-move (- right-move left-move))
          (setf left-move 0))
    (send self :frame-poly 
          (list
           (list (+ x-out center-out) 
                 (+ y-out below-out))
           '(0 10)
           (list right-move 0) 
           (list 0 top-move)
           (list (- left-move)  0)
           '(0 5) '(-3 -3) '(6 0) '(-3 3)) nil)
    (when new 
          (let ((to-list 
                 (select (send self :connection-list) icon-number-out))
                )
            (setf to-list (remove 'nil (combine to-list icon-number-in)))
            (setf (select (send self :connection-list) icon-number-out)
                  to-list)))))

(defmeth guidemap-proto :do-click (x y m1 m2)
  (send help-overlay-proto :do-click x y m1 m2 self)
  (let* ((n (send self :num-icons))
         (redraw-order (send self :redraw-order))
         (i nil)
         (iconx (send self :x))
         (icony (send self :y))
         (icon-type nil)
         (new-selected-icon nil)
         (newxy nil)
         (ix nil)
         (iy nil)
        )
    (dotimes (j n)
        (setf i  (select redraw-order (- n 1 j)))
        (setf ix (select iconx i))
        (setf iy (select icony i))
        (when (and (> x ix) (< x (+ ix 45)) (> y iy) (< y (+ iy 13)))
           (setf new-selected-icon (select (send self :icon-list) i))
           (setf icon-type (send new-selected-icon :icon-type))
              (when (= icon-type 6) ;guide icon
                    (if (< x (+ ix 23))
                        (send self :show-help   new-selected-icon)
                        (send self :icon-action new-selected-icon)))))))

(defmeth guidemap-proto :icon-action (icon)
  (when (equal "selected" (send icon :icon-state))
        (let* ((to-icon-num-list nil)
               (to-icon nil)
               (icon-list (send self :icon-list))
               (to-icon-type nil)
               (old-selected-icon (send self :selected-icon))
               (result nil))
          (send icon :icon-state "grey")
          (send *vista* :button-down t)
          (unwind-protect 
          (send self :selected-icon (position icon icon-list))
          (setf result (send icon :do-action)) 
(when investigate 
(format t "In ICON-ACTION - working on icon implications.~%")
(check)(break))
          (when (not result) 
                (send icon :icon-state "selected")
                (send self :selected-icon old-selected-icon))
          (when result
             (when (and (send *vista* :button-down)
                        (send *vista* :delay-update))
                   (send *guidemap* :toolbar nil))
             (dotimes (i (send self :num-icons))
               (when (equal icon (select icon-list i))
                 (setf to-icon-num-list 
                       (select (send self :connection-list) i))
                 (when (first to-icon-num-list)
                     (dolist (j to-icon-num-list)
                      (setf to-icon-type 
                            (send (select icon-list j) :icon-type))
                      (setf to-icon (select icon-list j))  
(when investigate (format t "Connected Icon ~G~%" j) (break))
                      (cond 
                        ((and (= to-icon-type 7) ;to icon is 'and' icon
                              (equal " " (send to-icon :title))) 
(when investigate (format t "AND Icon~%") (break))
                         (when (= (send to-icon :num-in-connected-now) 0)
                               (send to-icon :num-in-connected-now
                                     (send to-icon :num-in-connections)))
                         (send to-icon :num-in-connected-now
                               (- (send to-icon :num-in-connected-now) 1))
                         (when (= (send to-icon :num-in-connected-now) 0)
                               (dolist 
                                (k (select (send self :connection-list) j))
                                (send (select icon-list k)
                                      :icon-state "selected"))))
                        ((and (= to-icon-type 7) ;to icon is 'auto' icon
                              (not (equal " " (send to-icon :title))))
                         (if (and (send *vista* :button-down)
                                  (send *vista* :delay-update))
                             (send *vista* :store-return-info
                                   t (send *vista* :guidemap-number))
                             (send to-icon :do-action)))
                        (t ;to icon is not 'and' icon
                         (send (select icon-list j) :icon-state "selected")))
                        ))))
                (when (and (send *vista* :button-down)
                           (send *vista* :delay-update))
                      (send *guidemap* :toolbar t)
(when investigate 
(format t "In ICON-ACTION, finishing delayed RETRIEVE & UPDATE~%")
(check)(break))
                      (send *guidemap* :retrieve-and-update-guidemap 
                            (- (length (send *vista* :guidemap-slots)) 1))
                      (send self :buffer-to-screen))
                (when (send *vista* :delay-return)
(when investigate 
(format t "In ICON-ACTION, finishing delayed RETURN~%")
(check)(break))
                      (send *guidemap* :return-to-parent-guidemap))
                (send *vista* :delay-update nil)
                (send *vista* :button-down nil)
                (when (send self :instant-return)
                      (send self :return-to-parent-guidemap))
                )))))

;;###########################################################################
;;functions for converting a saved expert workmap file into a novice guidemap
;;###########################################################################
        
(defun guidance (&optional string)
"Args: &OPTIONAL STRING
Function used by guidance menu items to show a guidemap. Gives data guidance when STRING is DATA, model guidance when STRING is MODEL, workmap guidance otherwise."
  (let ((object nil)
        ) 
(when investigate
(format t "Entering GUIDANCE - string: ~a~%" string)
(check)(break))
    (send *vista* :delay-update nil)
    (when (and (equal string "data") (not current-data)) (setf string nil))
    (when (and (equal string "model") (not current-model)) (setf string nil))
    (cond 
      ((equal string "data")
       (setf object current-data)
       (if (send object :dob-parents)
           (cond 
             ((send object :ways)
              (send *vista* :guidemap-name "save-tab"))
             ((send object :matrices)
              (send *vista* :guidemap-name "save-mat"))
             (t (send *vista* :guidemap-name "save-dat")))
           (cond 
             ((send object :ways)
              (send *vista* :guidemap-name "table-da"))
             ((send object :matrices)
              (send *vista* :guidemap-name "matrix-d"))
             (t (send *vista* :guidemap-name "data-ana")))))
      ((equal string "model")
       (setf object current-model)
       (send *vista* :guidemap-name "model"))
      (t
       (setf object *workmap*)
       (send *vista* :guidemap-name "load-dat")))
    (when (not object) (setf object *workmap*)) 
(when investigate
(format t "Guidance: system gm= ~g, dob/mob gm = ~g~%object =~a"
(send *vista* :guidemap-number) (send object :guidemap-number) object)
(check)(break))
;    (send *vista* :guidemap-number (send object :guidemap-number))
    (when (not (send object :guidemap-number))
          (send *vista* :need-new-guidemap t)) 
    (let ((guidemap-name (send *vista* :guidemap-name))
          (guidemap-number (send object :guidemap-number)))
      (when (not *guidemap*)
            (setf *guidemap* (guidemap-window))
            (setf *hidden-guidemap* (guidemap-window)))
      (when (send *vista* :need-new-guidemap)
            (send *guidemap* :load-and-store-guidemap guidemap-name)
            (setf guidemap-number 
                  (- (length (send *vista* :guidemap-slots)) 1))
            (send object :guidemap-number guidemap-number))
(when investigate
(format t "In GUIDANCE - entering RETRIEVE & UPDATE:~%") 
(check)(break))
      (when (not (send *vista* :delay-update))
            (send *guidemap* :retrieve-and-update-guidemap guidemap-number))
      (send *guidemap* :toolbar t)
  ;    (send *vista* :guide-window-size (send *guidemap* :size))
  ;    (send *vista* :guide-window-location (send *guidemap* :location))
      (when (not (send *vista* :guidemap))
#+macintosh (send *workmap* :size 243 280)
#-macintosh (send *workmap* :size 246 280)
            (send *vista* :guidemap t)
            (send *guidemap* :gui t)))
    (send *vista* :need-new-guidemap nil))
  (send *guidemap* :show-window))

(defmeth guidemap-proto :load-and-store-guidemap (guidemap-name)
"Args: guidemap-name
Function to load guidemap file.  The file sends slot information to the hidden guidemap object.  This function then stores the slot information for later use. The hidden guidemap object is used so that the visible guidemap does not change." 
(when investigate
(format t "Load & Store GuideMap ~a~%" guidemap-name))
  (when (send *vista* :button-down) (send *vista* :delay-update t))
  (send *hidden-guidemap* :load-guidemap guidemap-name)
  (send *vista* :store-return-info nil nil)
  (send *vista* :store-slot-info (list (send *hidden-guidemap* :slot-list))))

(defmeth guidemap-proto :retrieve-and-update-guidemap (guidemap-number)
"Args: guidemap-number
Function to retrieve a stored guidemap and update the guidemap window to show the new guidemap."
(when investigate 
(format t "Ready to Retrieve & Update with ~g when system now has ~g~%" 
guidemap-number (send *vista* :guidemap-number))
(check)(break))
  (when (not (eq guidemap-number (send *vista* :guidemap-number)))
(when investigate
(format t "Retrieve & Update GuideMap # ~f~%" guidemap-number))
        (send *vista* :guidemap-number guidemap-number)
 ;       (send self :start-buffering)
        (send self :instant-return 
              (select (send *vista* :instant-return) guidemap-number))
        (send self :update-slot-info 
              (select (send *vista* :guidemap-slots) guidemap-number))
        (when (not (send self :instant-return)) (send self :redraw))
   ;     (send self :buffer-to-screen)
        ))

(defmeth guidemap-proto :link-to-new-guidemap (string)
"Args: String
Used by Link button to get new guidemap contained in file STRING."
(when investigate 
(format t "GuideMap Ancestors: ~f~%" 
(send current-object   :guidemap-ancestors)))
  (send *guidemap* :load-and-store-guidemap
        (blanks-to-dashes (subseq string 0 (min 8 (length string)))))
  (send current-object :guidemap-ancestors
        (append (list (send current-object :guidemap-number))
                (send current-object :guidemap-ancestors) ))
  (send current-object :guidemap-number 
        (- (length (send *vista* :guidemap-slots)) 1))
(when investigate 
(format t "GuideMap Ancestors: ~f~%" 
(send current-object :guidemap-ancestors)))
  t)

(defmeth guidemap-proto :delay-return-to-parent-guidemap ()
"Args: None
Prepares for delayed return to parent of present guidemap. Used by Return button."
  (send *vista* :delay-return t))

(defmeth guidemap-proto :return-to-parent-guidemap ()
"Args: None
Returns to parent of present guidemap. Used by icon-action."
(when investigate 
  (format t "In RETURN-TO-PARENT-GM - ABOUT TO ENTER R&U ~%GuideMap Ancestors: ~f~%" (send current-object :guidemap-ancestors))
(check)(break))
  (cond 
    ((send current-object :guidemap-ancestors)
     (send *guidemap* :retrieve-and-update-guidemap
           (first (send current-object :guidemap-ancestors)))
     (send current-object :guidemap-number 
           (first (send current-object :guidemap-ancestors)))
     (send current-object :guidemap-ancestors
           (rest (send current-object :guidemap-ancestors))))
    (t
     (setcd (first (send current-object :dob-parents)))))
  (send *vista* :delay-return nil)
  t)

(defmeth guidemap-proto :goto-new-object (string)
"Args: STRING
Switches current-object to the object whose title is STRING. STRING is either 1) the title of the goto data object; 2) the generic title of the goto data object (the portion of the title up to the first dash) when the goto data object is a child of another object; or 3) 'model', for goto model objects." 
  (cond 
    ((equal "model" (string-downcase string)) (setcm current-model))
    (t (let* ((child-title nil)
              (position nil)
              (object nil)
              (children (send current-object :dob-children))
              (child nil)
              (child-icon nil)
              (num-children (length children))
              (goto-title nil))
         (when children
               (dotimes (i num-children)
                        (setf child (select children i))
                        (setf child-icon
                              (select (send *workmap* :icon-list)
                                      (1- (send child :icon-number))))
                        (setf child-title (send child-icon :title))
                        (setf position (search-string "-" child-title))
                        (when position
                              (setf goto-title 
                                    (subseq child-title 0 position))
                              (format t "Found: ~a~%" goto-title)
                              (when (equal goto-title string) 
                                    (setf object 
                                          (eval (intern (string-upcase 
                                                         child-title))))))))
         (when (not object) 
               (setf object (eval (intern (string-upcase string)))))
         (setcd object)))))

(defmeth guidemap-proto :created-data ()
"Args: None
Updates title of attached guidemap goto button when data are created by a Create-Data guidemap button."
  (let* ((active-button-number (send self :selected-icon));not updated
         (to-icon-num-list 
          (select (send self :connection-list) active-button-number))
         (to-icon nil)
         (j (1- (length (send *workmap* :icon-list))))
         (newtitle nil)
         ) 
    (dolist (i to-icon-num-list)
            (setf to-icon (select (send self :icon-list) i))
            (when (equal "GoTo:New Data" (send to-icon :title))
                  (setf newtitle (strcat "GoTo:"
                       (send (select (send *workmap* :icon-list) j) :title))))
            (when (equal "Link:Model" (send to-icon :title))
                  (setf newtitle (strcat "Link:"
                       (send current-model :model-abbrev))))
            (when newtitle
                  (send to-icon :title newtitle)
                  (setf (select (send self :icon-title) i) newtitle)
                  (send to-icon :make-action)
                  (send self :redraw)
                  (setf newtitle nil))))
  t)

(defmeth guidemap-proto :update-slot-info (slot-list)
"Args: SLOT-LIST
Updates guidemap slots from SLOT-LIST"
  (send self :NUM-ICONS (select slot-list 1))
  (send self :CONNECTION-LIST (select slot-list 2))
  (send self :X (select slot-list 3))
  (send self :Y (select slot-list 4))
  (send self :ICON-TYPE (select slot-list 5))
  (send self :ICON-TITLE (select slot-list 6))
  (send self :ICON-NUMBER-LIST (select slot-list 7))
  (send self :REDRAW-ORDER (select slot-list 8))
  (send self :SELECTED-ICON (select slot-list 9))
  (send self :TOOLBAR t)
  (send self :NUM-DATA-ICONS (select slot-list 11))
  (send self :NUM-MODEL-ICONS (select slot-list 12))
  (send self :DATA-ICON-NUMBER-LIST (select slot-list 13))
  (send self :MODEL-ICON-NUMBER-LIST (select slot-list 14))
  (send self :SELECTED-DATA-ICON (select slot-list 15))
  (send self :PREVIOUSLY-SELECTED-DATA-ICON (select slot-list 16))
  (send self :NUM-DATA-MENU-ITEMS (select slot-list 17))
  (send self :NUM-MODEL-MENU-ITEMS (select slot-list 18))
  (send self :ACTIVE-BUTTON-LIST (select slot-list 19))
  (send self :ICON-LIST (select slot-list 20))
  (send self :DATA-ICON-LIST (select slot-list 21))
  (send self :MODEL-ICON-LIST (select slot-list 22))
  (send self :TITLE (select slot-list 0))
  )

(defmeth guidemap-proto :slot-list ()
"Args: none
Returns a list of the current guidemap slot information"
  (list
   (send self :TITLE) 
   (send self :NUM-ICONS) 
   (send self :CONNECTION-LIST)
   (send self :X)
   (send self :Y)
   (send self :ICON-TYPE)
   (send self :ICON-TITLE)
   (send self :ICON-NUMBER-LIST)
   (send self :REDRAW-ORDER)
   (send self :SELECTED-ICON)
   (send self :TOOLBAR)
   (send self :NUM-DATA-ICONS) 
   (send self :NUM-MODEL-ICONS) 
   (send self :DATA-ICON-NUMBER-LIST)
   (send self :MODEL-ICON-NUMBER-LIST)
   (send self :SELECTED-DATA-ICON)
   (send self :PREVIOUSLY-SELECTED-DATA-ICON) 
   (send self :NUM-DATA-MENU-ITEMS)
   (send self :NUM-MODEL-MENU-ITEMS) 
   (send self :ACTIVE-BUTTON-LIST)
   (send self :ICON-LIST)
   (send self :DATA-ICON-LIST)
   (send self :MODEL-ICON-LIST)))

(defmeth guidemap-proto :load-guidemap (guidemap-name)
"Args: GUIDEMAP-NAME
Loads guidemap information file GUIDEMAP-NAME and generates icon information."
  (let ((result (load (strcat *guide-dir-name* 
                              (string-downcase guidemap-name))))
        (ab-list nil)
        (icon-objectid-list (send self :icon-list)))
    (when result 
          (setf ab-list (send self :active-button-list))
          (dotimes (i (send self :num-icons))
                   (cond 
                     ((= (select (send self :icon-type) i) 7); 'and' icon
                      (send (select icon-objectid-list i) 
                            :icon-state "normal"))
                     (t
                      (send (select icon-objectid-list i) 
                            :icon-state "grey"))))
          (cond 
            ((not ab-list)
             (send (select icon-objectid-list 0) :icon-state "selected"))
            (t
             (dolist (i ab-list)
                     (send (select icon-objectid-list i)
                           :icon-state "selected"))))
          (dotimes (i (send self :num-icons))
                  (send (select icon-objectid-list i) :window *guidemap*))
          (send self :instant-return nil)
          )
    (when (not result) 
          (error (format nil "No GuideMap File named~% ~a" 
                         (strcat *guide-dir-name* guidemap-name))))
    result))

(defun guidemap-window ()
"Args: None
GuideMap-Window object constructor function.  This function creates a guidemap window, but does not create a map.  The map is created by the code in the guidemap file. Returns window object-id."
  (let ((object (send guidemap-proto :new 2
            :title "ViSta GuideMap"
            :show nil)))
    (menus t)
    (apply #'send object :size (send *vista* :guide-window-size))
    (apply #'send object :location (send *vista* :guide-window-location))
    (send object :menu nil)
    (send object :toolbar t)
    object))

(defun make-icon (w x y title icon-type)
  (let ((obj nil)
        )
    (cond
      ((= icon-type 1) 
       (setf obj (send dob-icon-proto   :new w x y 25 32 :title title)))
      ((= icon-type 2) 
       (setf obj (send tool-icon-proto  :new w x y 45 13 :title title)))
      ((= icon-type 3) 
       (setf obj (send mob-icon-proto   :new w x y 25 32 :title title)))
      ((= icon-type 4) 
       (setf obj (send dib-icon-proto   :new w x y 25 32 :title title)))
      ((= icon-type 5)
       (setf obj (send tab-icon-proto   :new w x y 25 32 :title title)))
      ((= icon-type 6)
       (setf obj (send guide-icon-proto :new w x y 45 13 :title title)))
      ((= icon-type 7)
       (setf obj (send and-icon-proto   :new w x y 45 13 :title title)))
      )
      obj))

;;###########################################################################
;;define help button overlay prototype and methods
;;###########################################################################

(defproto help-overlay-proto '() () graph-overlay-proto)

(defmeth help-overlay-proto :redraw ()
  (let* ((graph (send self :graph))
         (scroll (send graph :scroll))
         (scrollx (first scroll))
         (scrolly (second scroll))
         (tw (send graph :text-width "Help")))
    (send graph :draw-color 'white)
    (send graph :paint-rect (+ scrollx 6)  (+ scrolly 3) (+ tw 22) 16)
    (send graph :draw-color 'black)
    (send graph :frame-rect (+ scrollx 6)  (+ scrolly 3) (+ tw 22) 16)
    (send graph :frame-rect (+ scrollx 10) (+ scrolly 6) 10 10)
    (send graph :draw-string "Help" (+ scrollx 24) (+ scrolly 16))))

(defmeth help-overlay-proto :do-click (x y m1 m2 graph)
  (let* ((scroll (send graph :scroll))
         (scrollx (first scroll))
         (scrolly (second scroll))
         (tw (send graph :text-width "Help")))
  (when (and (< (+ scrollx 6) x (+ scrollx tw 28))  10 20
             (< (+ scrolly 3) y (+ scrolly 18)))  6 16
        (send graph :show-help graph))))

(defmeth workmap-proto :show-help (icon)
  (let* ((title (send icon :title))
         (i (min (list 8 (length title))))
         (help-file-name title)
         (w (send *vista* :help-window-object))
        )
    (when (equal "GoTo" (subseq title 0 4)) 
          (setf title "GoTo")
          (setf help-file-name title)
          (setf i 4))
    (when (and (> i 4) (equal ":" (subseq title 4 5)) )
          (setf help-file-name (subseq title 5 (length title)))
          (when (> (length help-file-name) 8)
                (setf help-file-name (subseq help-file-name 0 8)))
          (when (> i (length help-file-name))
                (setf i (length help-file-name)))
          )
    (setf help-file-name 
          (strcat *help-dir-name* 
             (string-downcase (subseq (blanks-to-dashes help-file-name) 0 i)) ".hlp"))
#+macintosh (file-to-window help-file-name title w)
#-macintosh (file-to-stream help-file-name title)
    ))

(setf investigate nil)
;(load (strcat *vista-dir-name* "gm-debug"))
(provide "guidemap")
