(defmeth morals-spreadplot-supervisor-proto :create-plots () 
  (let* (
         (i 0)
         (j 0)
         (k 0)
         (iteration-list nil)
         (iter-list nil)
         (rsq-beta-list nil)
         (a 0)
         (model (send (send self :model) :morals-model))
         (dv (select (send (send self :model) :variables) 
                     (send (send self :model) :dv)))
         (pred (strcat "OLS Predicted " dv))
         (count (+ (send model :count) 1))
         (rsq-list (send model :rsq-list))
         (lin-reg (send model :lin-reg-model))
         (rsqp (rsq-plot self :variable-labels '("Iterations"
                                            "R-Square/Betas")
                              :title "R-Square/Betas"
                              :go-away t
                              :show nil))
         (rp1 (residual-plot self (send model :fit-values)
                            (send model :studentized-residuals)
                            :go-away t
                            :show nil :title "Residuals 1"
                            ))
         (ip1 (influence-plot self (send model :fit-values)
                            (send model :cooks-distances)
                            :title "Influence 1"
                            :go-away t
                            :show nil))
         (tp1 nil)
         (rp2 (residual-plot self (send model :fit-values)
                            (send model :raw-residuals)
                            :show nil
                            :go-away t
                            :title "Residuals 2"))
         (ip2 (influence-plot self (send model :fit-values)
                            (send model :leverages)
                            :title "Influence 2"
                            :go-away t
                            :show nil))
         (lr (lin-reg-plot self (send lin-reg :Y) (send lin-reg :fit-values) 
                         :title "Linear Regression"
                         :go-away t
                         :show nil))
         (var-list (send var-list-proto :new self))
         (obs-list (send obs-list-proto :new self))
         )
    
    (if (= (send model :count) 0)
        (setf tp1 (lsmt-plot (send model :YRaw) (send model :Y)
                             (send model :fit-values) self 
                             :title "LS Monotone Transformation"
                             :show nil))
        (setf tp1 (lsmt-plot (send model :YRaw) (send model :Y)
                             (send model :YHat) self
                             :title "LS Monotone Transformation"
                             :show nil)))
    (send rp1 :size 270 270)
    (send rp1 :location 7 309)
    (send rp1 :adjust-to-data)
    (send rp1 :linked t)
    (send rp1 :abline 0.0 0.0)
    (send rp1 :add-overlay (make-overlay2 rp1 self))
    (send rp1 :showing t)
    (send rp2 :size 270 270)
    (send rp2 :location 279 309)
    (send rp2 :adjust-to-data)
    (send rp2 :linked t)
    (send rp2 :abline 0.0 0.0)
    (send rp2 :add-overlay (make-overlay2 rp2 self))
    (send rp2 :showing t)
    (send ip1 :size 270 270)
    (send ip1 :location 279 19)
    (send ip1 :adjust-to-data)
    (send ip1 :linked t)
    (send ip1 :add-overlay (make-overlay2 ip1 self))
    (send ip1 :showing t)
    (send ip2 :size 270 270)
    (send ip2 :location 551 19)
    (send ip2 :adjust-to-data)
    (send ip2 :linked t)
    (send ip2 :add-overlay (make-overlay2 ip2 self))
    (send ip2 :showing t)
    (send tp1 :variable-label '(0 1)
          (list dv pred))
    (send tp1 :size 270 270)
    (send tp1 :location 7 19)
    (send tp1 :linked t)
    (send tp1 :add-overlay (make-overlay2 tp1 self))
    (send tp1 :redraw)
    (send tp1 :adjust-to-data)
    (send tp1 :showing t)
    (send tp1 :title "OLS Regression")
    (send lr :abline 0 1)
    (send lr :size 270 270)
    (send lr :location 7 19)
    (send lr :linked t)
    (send lr :add-overlay (make-overlay2 lr self))
    (send var-list :size 270 270)
    (send var-list :location 551 19)
    (send var-list :linked t)
    (send obs-list :size 270 270)
    (send obs-list :location 551 309)
    (send obs-list :linked t)
    (send obs-list :showing t)
    
    (when (> (send model :count) 0)
          (dotimes (k count)
             (setf iteration-list (append iteration-list 
                             (list (repeat k (length (first 
                                             (send model :standardized-beta-list)))))))))
    (send rsqp :clear-points)
    (send rsqp :clear-lines)
    (dotimes (k count)
             (setf iteration-list (append iteration-list 
                             (list (repeat k (length (first 
                                   (send model :standardized-beta-list))))))))
    (dotimes (j (length (first (send model :standardized-beta-list)))) 
           (dotimes (i (length (send model :standardized-beta-list)))  
                    (setf rsq-beta-list (append rsq-beta-list (list 
                                  (select (select 
                                  (send model :standardized-beta-list) i) 
                                          j))))
                    (setf iter-list (append iter-list (list (select (select
                                    iteration-list i) j)))))
           (send rsqp :add-lines iter-list rsq-beta-list)
           (send rsqp :add-lines (iseq count) rsq-list)
           (setf i 0)
           (setf iter-list nil)
           (setf rsq-beta-list nil))
    (send rsqp :add-points (list (repeat (send model :count) 
            (length (combine (last (send model :standardized-beta-list)))))
                    (combine (last (send model :standardized-beta-list))))
            :point-labels (select (send (send self :model) :variables)
                                (send (send self :model) :iv)))
    (when (equalp (send model :count) 0)
          (send rsqp :add-points (list 0) (list (send model :r-squared))))
    (when (> (send model :count) 0)
          (send rsqp :add-points (list (send model :count)) 
                     (list (select (send model :rsq-list) 
                                   (send model :count)))
                     :point-labels (list "RSQ")))

    (send rsqp :adjust-to-data)
    (send rsqp :location 279 19)
    (send rsqp :size 270 270)
    (send rsqp :linked t)
    (send rsqp :add-overlay (make-overlay2 rsqp self))
    (when (not (send model :use-robust))
          (send self :infl-type1 "LR-Cooks")
          (send self :infl-type2 "LR-Lev")
          (send self :resid-type1 "LR-Raw")
          (send self :resid-type2 "LR-Student"))
    (when (equalp (send model :use-robust) 2)
          (send self :infl-type1 "MR-Cooks")
          (send self :infl-type2 "MR-Lev")
          (send self :resid-type1 "MR-Raw")
          (send self :resid-type2 "MR-Student")
          (send model :update-transformation-plot)
          (send ip2 :hide-window)
          (send ip2 :showing nil)
          (send rp2 :hide-window)
          (send rp2 :showing nil)
          (send var-list :showing t)
          (send var-list :show-window)
          (send rsqp :showing t)
          (send rsqp :show-window)
          (send ip1 :location 279 309))
    (when (equalp (send model :use-robust) 3)
          (send self :infl-type1 "RR-Cooks")
          (send self :infl-type2 "RR-Lev")
          (send self :resid-type1 "RR-Raw")
          (send self :resid-type2 "RR-Student")
          (send self :create-robust-plot)
          (send self :create-robust-reg-plot)
          (send ip2 :hide-window)
          (send ip2 :showing nil)
          (send tp1 :hide-window)
          (send tp1 :showing nil))
    (send self :rsq-beta-plot rsqp)    
    (send self :residual-plot1 rp1)
    (send self :residual-plot2 rp2)
    (send self :influence-plot1 ip1)
    (send self :influence-plot2 ip2)
    (send self :transformation-plot tp1)
    (send self :lin-reg-plot lr)
    (send self :var-list var-list)
    (send self :obs-list obs-list)
    (send self :update-residual-plot)
    (send self :update-influence-plot)
    ))

