;;########################################################################
;; unimob.lsp
;; code for univariate analysis model object (continued in unimob1)
;; Copyright (c) 1995-6 by Forrest W. Young
;;########################################################################


;1 define the prototype inheriting from mv-model-object-proto

(defproto univar-model-object-proto 
  '(var var1 var2 mu sigma t-p-level direction ci-level  
    n-ways t-value t-df t-ci-limits paired std-err-diff std-err
    pooled-variance nonpar nonpar-z nonpar-p mann-whitney-sum 
    var1-label var2-label)
  () mv-model-object-proto)

;2 define the slot accessor methods

(defmeth univar-model-object-proto :var (&optional (list nil set))
"Message args: (&optional list)
Sets or retrieves the data that are analyzed.  These are the sample values (when there is one sample), sample differences (for two paired samples) or values for both samples (for two independent samples)." 
  (if set (setf (slot-value 'var) list))
  (slot-value 'var))

(defmeth univar-model-object-proto :var1 (&optional (list nil set))
"Message args: (&optional list)
Sets or retrieves values for the first sample." 
  (if set (setf (slot-value 'var1) list))
  (slot-value 'var1))

(defmeth univar-model-object-proto :var2 (&optional (list nil set))
"Message args: (&optional list)
Sets or retrieves values for the second sample, if there is one." 
  (if set (setf (slot-value 'var2) list))
  (slot-value 'var2))

(defmeth univar-model-object-proto :mu (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the population mean." 
  (if set (setf (slot-value 'mu) number))
  (slot-value 'mu))

(defmeth univar-model-object-proto :sigma (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the population standard-deviation." 
  (if set (setf (slot-value 'sigma) number))
  (slot-value 'sigma))

(defmeth univar-model-object-proto :t-df (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the degrees of freedom." 
  (if set (setf (slot-value 't-df) number))
  (slot-value 't-df))

(defmeth univar-model-object-proto :t-value (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the t-value." 
  (if set (setf (slot-value 't-value) number))
  (slot-value 't-value))

(defmeth univar-model-object-proto :t-p-level (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the p-level for the t-value." 
  (if set (setf (slot-value 't-p-level) number))
  (slot-value 't-p-level))

(defmeth univar-model-object-proto :direction (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the direction. Negative or positive for a one-tailed directional test, zero or nil for a two-tailed test." 
  (if set (setf (slot-value 'direction) number))
  (slot-value 'direction))

(defmeth univar-model-object-proto :ci-level (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the t value confidence interval level." 
  (if set (setf (slot-value 'ci-level) number))
  (slot-value 'ci-level)) 

(defmeth univar-model-object-proto :n-ways (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the number of ways of table data (0 for mv data)." 
  (if set (setf (slot-value 'n-ways) number))
  (slot-value 'n-ways)) 

(defmeth univar-model-object-proto :t-ci-limits (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the t value confidence interval limits." 
  (if set (setf (slot-value 't-ci-limits) number))
  (slot-value 't-ci-limits)) 

(defmeth univar-model-object-proto :std-err-diff (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the standard error of the difference of two independent samples." 
  (if set (setf (slot-value 'std-err-diff) number))
  (slot-value 'std-err-diff))

(defmeth univar-model-object-proto :std-err (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the standard error." 
  (if set (setf (slot-value 'std-err) number))
  (slot-value 'std-err))

(defmeth univar-model-object-proto :pooled-variance 
  (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the pooled variance for two independent samples." 
  (if set (setf (slot-value 'pooled-variance) number))
  (slot-value 'pooled-variance))

(defmeth univar-model-object-proto :paired (&optional (logical nil set))
"Message args: (&optional logical)
Sets or retrieves whether situation is paired samples." 
  (if set (setf (slot-value 'paired) logical))
  (slot-value 'paired))

(defmeth univar-model-object-proto :nonpar (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the value of the nonparametric test statistic (Wilcoxon T or Mann-Whitney U)." 
  (if set (setf (slot-value 'nonpar) number))
  (slot-value 'nonpar))

(defmeth univar-model-object-proto :nonpar-z (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the value of the large sample normal approximation to the nonparametric statistic." 
  (if set (setf (slot-value 'nonpar-z) number))
  (slot-value 'nonpar-z))

(defmeth univar-model-object-proto :nonpar-p (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the significance level of the large sample normal approximation to nonparametric statistic." 
  (if set (setf (slot-value 'nonpar-p) number))
  (slot-value 'nonpar-p))

(defmeth univar-model-object-proto :mann-whitney-sum 
  (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the mann-whitney sum statistic." 
  (if set (setf (slot-value 'mann-whitney-sum) number))
  (slot-value 'mann-whitney-sum))

(defmeth univar-model-object-proto :var1-label 
  (&optional (string nil set))
"Message args: (&optional string)
Sets or retrieves the label of the first variable." 
  (if set (setf (slot-value 'var1-label) string))
  (slot-value 'var1-label))

(defmeth univar-model-object-proto :var2-label 
  (&optional (string nil set))
"Message args: (&optional string)
Sets or retrieves the label of the second variable." 
  (if set (setf (slot-value 'var2-label) string))
  (slot-value 'var2-label))


;3 Vista System Methods

(defmeth univar-model-object-proto :options ()
  (if (= (send self :n-ways) 0)
      (send self :dependent-dialog)
      (send self :independent-dialog)))

(defmeth univar-model-object-proto :independent-dialog ()
"Message args: none
For a two independent samples, presents dialog box to get univariate analysis options if :dialog t is used on the model constructor statement. Returns nil if :dialog nil, if dialog values are bad, or if dialog is canceled. Returns dialog values otherwise. Dialog values are checked for accuracy and error messages are produced if values are bad."
  (when (send self :dialog)
        (let* ((n-ways (send self :n-ways))
               (title (send text-item-proto :new "Univariate Analysis: Two Independent Samples"))
               (text1 (send text-item-proto :new "Ho: Difference in Population Means ="))
               (mu    (send edit-text-item-proto :new "        0.00"))
               (text2 (send text-item-proto :new "H1: (alternative hypothesis)"))
               (direc (send choice-item-proto :new 
                            (list "Difference in Sample Means LT Difference in Pop Means (1 tail)"
                                  "Difference in Sample Means NE Difference in Pop Means (2 tails)"
                                  "Difference in Sample Means GT Difference in Pop Means (1 tail)")
                            :value 1))
               (text3 (send text-item-proto :new 
                            "Confidence Interval Level ="))
               (cilev (send edit-text-item-proto :new ".95"))
               (ok (send modal-button-proto :new "OK"
                         :action #'(lambda () 
                                     (list (- (send direc :value) 1)
                                           (read-from-string 
                                            (send mu    :text) nil)
                                           (read-from-string 
                                            (send cilev :text) nil)
                                           ))))
               (cancel (send modal-button-proto :new "Cancel"))
               (dialog (send modal-dialog-proto :new
                             (list title 
                                   (list text1 mu) 
                                   text2 direc
                                   (list text3 cilev)
                                   (list ok cancel)) :default-button ok))
               (result (send dialog :modal-dialog))
               (error nil))
          (when result
                (when (not (numberp (second result)))
                      (error-message "The Population Mean must be a number.")
                      (setf error t))
                (when (or (not (numberp (third result)))
                          (>= (third result) 1) (<= (third result) 0))
                      (error-message "The Confidence Level must be a number between 0 and 1, but not including 0 or 1.")
                      (setf error t))
                (when error (setf result nil)))
          (when result
                (send self :direction (first result))
                (send self :mu (second result))
                (send self :ci-level (third result))
                (send self :var (send current-data :data))
                (send self :var1-label 
                      (first (first (send current-data :classes))))
                (send self :var2-label 
                      (second (first (send current-data :classes))))
                (send self :var1 (first (send current-data :data)))
                (send self :var2 (second (send current-data :data))))
          )))

(defmeth univar-model-object-proto :dependent-dialog ()
"Message args: none
For a single sample, or for two dependent samples, presents dialog box to get univariate analysis options if :dialog t is used on the model constructor statement. Returns nil if :dialog nil, if dialog values are bad, or if dialog is canceled. Returns dialog values otherwise. Dialog values are checked for accuracy and error messages are produced if values are bad."
  (when (send self :dialog)
        (let* ((title (send text-item-proto :new "Univariate Analysis: One Sample or Two Paired Samples" :location '(55 5)))
               (title2 (send text-item-proto :new "1) Provide details for t (or Z) test:"))
               (text1 (send text-item-proto :new "Ho: Population Mean ="))
               (mu    (send edit-text-item-proto :new "        0.00"))
               (text2 (send text-item-proto :new "H1: (alternative hypothesis)"))
               (direc (send choice-item-proto :new 
                            (list "Sample Mean LT Pop Mean (1 tail)"
                                  "Sample Mean NE Pop Mean (2 tails)"
                                  "Sample Mean GT Pop Mean (1 tail)")
                            :value 1))
               (text2a (send text-item-proto :new (format nil 
                       "Population Std. Dev. =~% (only if known)")))
               (popvar (send edit-text-item-proto :new "              "))
               (text3 (send text-item-proto :new "Confidence Interval Level ="))
               (cilev (send edit-text-item-proto :new ".95"))
               (text4 (send text-item-proto :new 
                            "2) Select a variable:"))
               
               (var1  (send list-item-proto :new
                            (send current-data :active-variables '(numeric))
                            :size (list 170 80)))
               (text5 (send text-item-proto :new (format nil 
                      "3) Select 2nd variable:~%(only for paired samples)")))
               (var2  (send list-item-proto :new
                            (send current-data :active-variables '(numeric))
                            :size (list 170 80)))
               (ok (send modal-button-proto :new "OK"
                         :action #'(lambda () 
                                     (list (- (send direc :value) 1)
                                           (read-from-string 
                                            (send mu    :text) nil)
                                           (read-from-string 
                                            (send cilev :text) nil)
                                           (send var1 :selection)
                                           (send var2 :selection)
                                           (read-from-string
                                            (send popvar :text) nil)
                                           ))))
               (cancel (send modal-button-proto :new "Cancel"))
               (dialog (send modal-dialog-proto :new
                             (list title 
                                  (list (list title2
                                              (list text1 mu) 
                                              text2 direc
                                              (list text2a popvar)
                                              (list text3 cilev)
                                              (list ok cancel))
                                        (list text4 var1 text5 var2))
                                   )
                             :default-button ok))
               (var0 (send var1 :selection 0))
               (result (send dialog :modal-dialog))
               (error nil)
               ) 
          (when result
                (when (not (numberp (second result)))
                      (error-message "The Population Mean must be a number.")
                      (setf error t))
                (when (or (not (numberp (third result)))
                          (>= (third result) 1) (<= (third result) 0))
                      (error-message "The Confidence Level must be a number between 0 and 1, but not including 0 or 1.")
                      (setf error t))
                (when (not (fourth result)) 
                      (error-message "You must select a variable from the first variable list")
                      (setf error t))
                (when (fifth result)
                      (when (equal (fifth result) (fourth result))
                            (error-message "If you select a second variable, it must be different than the first.")
                            (setf error t)))
                (when (sixth result)
                      (when (not (numberp (sixth result)))
                            (error-message "If you specify a Population Variance, it must be a number.")
                            (setf error t)))
                (when error (setf result nil)))
          (when result
                (send self :direction (first result))
                (send self :mu (second result))
                (send self :ci-level (third result))
                (send self :var1-label 
                      (select (send current-data :active-variables '(numeric)) (fourth result)))
                (send self :var1 (col (send current-data :active-data-matrix '(numeric))
                                      (fourth result)))
                (cond 
                  ((fifth result) 
                   (send self :var2-label 
                      (select (send current-data :active-variables '(numeric)) (fifth result)))
                   (send self :var2 (col (send current-data :active-data-matrix '(numeric))
                                         (fifth result)))
                   (send self :var (- (send self :var1) (send self :var2)))
                   (send self :paired t))
                  (t (send self :var (send self :var1))))
                (when (sixth result)
                      (send self :sigma (sixth result)))
                )
          result)))

(defmeth univar-model-object-proto :analysis ()
"Args: none
Does analysis for the univariate analysis model."
  (if (= (send self :n-ways) 0)
      (send self :dependent-analysis)
      (send self :independent-analysis)))

(defmeth univar-model-object-proto :dependent-analysis ()
"Args: none
Does analysis for the univariate analysis model. Computes, for single sample or paired sample situations, the t-test or z-test value, p-level and confidence limits. When appropriate, also computes wilcoxon test."
  (send self :t-df (- (length (send self :var)) 1))
  (send self :t-value (send self :compute-t-or-z-value))
  (send self :t-p-level (send self :compute-t-or-z-p-level))
  (send self :t-ci-limits (send self :compute-t-or-z-ci-limits)) 
  (when (and (>= (send self :t-df) 7) (send self :paired)) 
        (send self :compute-wilcoxon))
  )

(defmeth univar-model-object-proto :independent-analysis ()
"Args: none
Does analysis for the univariate analysis model. Computes, for two independent sample situation, the t-test or z-test value, p-level and confidence limits. When appropriate, also computes Mann-Whitney test." 
  (send self :t-df (- (sum (mapcar #'length (send self :var))) 2))
  (send self :t-value (send self :compute-independent-t))
  (send self :t-p-level (send self :compute-t-or-z-p-level))
  (send self :t-ci-limits (send self :compute-t-or-z-ci-limits)) 
  (when (>= (min (mapcar #'length (send self :var))) 7) 
        (send self :compute-mann-whitney))
  )

(load (strcat *vista-dir-name* "unimob1"))